/* This file is part of PolyCalc.

   PolyCalc is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   any later version.

   PolyCalc is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PolyCalc.  If not, see <http://www.gnu.org/licenses/>.

   For more information about PolyCalc see project home
   <https://launchpad.net/polycalc>.

   Copyright 2012 PolyCalc development team */

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QMessageBox>
#include <QDebug>
#include <QFileDialog>
#include <QFileInfo>
#include <QtXml>
#include <QErrorMessage>

const QString version = "0.1.0";
const QString prjUrl = "launchpad.net/polycalc";
const QString helpUrl = "launchpad.net/polycalc";

class ButtonAction : public QObject {
  public:
    QString insert;
    QString set;
    bool send;
    ButtonAction(QString insert, QString set, bool send)
               : insert(insert), set(set),    send(send) {}
};

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    menuSignalMapper = new QSignalMapper(this);
    buttonSignalMapper = new QSignalMapper(this);
    scriptEngine = new QScriptEngine();
    settings     = new QSettings("PolyCalc development team", "PolyCalc");
    ui->setupUi(this);
    ui->dockWidget->hide();
    PopulateMenu();
    QString initLayoutFile("basic.cal");
    PopulateButtons(initLayoutFile);
    ui->lineEdit->insertItem(0,"");
    ui->lineEdit->setCurrentIndex(0);
    ui->lineEdit->lineEdit()->setAlignment(Qt::AlignRight);
    connect(ui->lineEdit->lineEdit(), SIGNAL(returnPressed(void)),
            this, SLOT(on_lineEdit_returnPressed(void)));
}

void MainWindow::PopulateMenu()
{
    // todo: don't use . dir, use ~/polycalc instead
    QDir dir(".");
    QStringList list = dir.entryList(QStringList("*.cal"));
    QStringListIterator iterator(list);
    while (iterator.hasNext()) {
        QString name = iterator.next();
        name.chop(4);
        QAction *action = new QAction(name, this);
        ui->menuOptions->addAction(action);
        menuSignalMapper->setMapping(action, name);
        connect(action, SIGNAL(triggered()),
                menuSignalMapper, SLOT (map()));
    }
    connect(menuSignalMapper, SIGNAL(mapped(const QString &)),
            this, SLOT(on_layoutMenuClicked(const QString &)));
}

void MainWindow::PopulateButtons(QString filename)
{
    static QGridLayout *rootGridWidget = NULL;
    QDomDocument doc("buttontable");
    QFile file(filename);
    QString errorMsg;
    int errorLine;
    int errorColumn;

    if (!file.open(QIODevice::ReadOnly)) {
        QErrorMessage errorMessage;
        errorMessage.showMessage(QString("Can not open file ") + filename);
        errorMessage.exec();
    }

    if (!doc.setContent(&file, true, &errorMsg, &errorLine, &errorColumn)) {
        file.close();
        QErrorMessage errorMessage;
        errorMessage.showMessage(QString("Error while processing file ") + filename
                                 + QString(" at line ") + QString::number(errorLine)
                                 + QString(", column ") + QString::number(errorColumn)
                                 + QString(": ") + errorMsg);
        errorMessage.exec();
    }
    file.close();

    if (rootGridWidget != NULL) {
        qDebug()<<"delete";
        ui->verticalLayout->removeItem(rootGridWidget);
        // todo: free childs also
        delete rootGridWidget;
        rootGridWidget=NULL;
    }
    QDomElement polycalc = doc.documentElement();
    QDomNode buttontable = polycalc.firstChild();
    QDomNode rootgrid = buttontable.firstChild();
    if(rootgrid.toElement().tagName() == "grid") {
        rootGridWidget = XMLGridNodeToQGridLayout(&rootgrid,
                                                  rootgrid.toElement().attribute("style",NULL));
        ui->verticalLayout->addLayout(rootGridWidget);
    }
    connect(buttonSignalMapper, SIGNAL(mapped(QObject *)),
            this, SLOT(on_anyButtonClicked(QObject *)));
}

QGridLayout* MainWindow::XMLGridNodeToQGridLayout(QDomNode *grid, QString parentStyle)
{
    int i = 0, j = 0;
    QGridLayout *gridWidget = new QGridLayout(this);
    for (QDomNode row = grid->firstChild();
         !row.isNull();
         row = row.nextSibling(), j++)
    {
        i = 0;
        QString rowStyle = row.toElement().attribute("style",NULL);
        for (QDomNode cell = row.firstChild();
             !cell.isNull();
             cell = cell.nextSibling(), i++)
        {
            int rowStretch = row.toElement().attribute("stretch", "1").toInt();
            int colStretch = cell.toElement().attribute("stretch", "1").toInt();
            QString cellStyle = cell.toElement().attribute("style",NULL);
            QString style = (cellStyle != NULL) ? cellStyle :
                            (rowStyle != NULL)  ? rowStyle  :
                                                  parentStyle;
            if(cell.toElement().tagName() == "button")
            {
                ButtonAction *buttonAction;
                QPushButton *button = new QPushButton(cell.toElement().attribute("caption"), this);
                button->setGeometry(0,0,40,25);
                button->setMinimumSize(40,25);
                button->setFocusPolicy(Qt::NoFocus);
                button->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
                button->setStyleSheet(style);
                gridWidget->addWidget(button, j, i);
                gridWidget->setRowStretch(j,rowStretch);
                gridWidget->setColumnStretch(i,colStretch);
                buttonAction = new ButtonAction(cell.toElement().attribute("insert", NULL),
                                                cell.toElement().attribute("set",    NULL),
                                                cell.toElement().attribute("send") == "true");
                buttonSignalMapper->setMapping(button, buttonAction);
                connect(button, SIGNAL(clicked()),
                        buttonSignalMapper, SLOT (map()));
            }
            if(cell.toElement().tagName() == "grid")
            {
                QGridLayout *childGridWidget = XMLGridNodeToQGridLayout(&cell, style);
                gridWidget->addLayout(childGridWidget, j, i);
            }
        }
    }
    return(gridWidget);
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::on_layoutMenuClicked(const QString &name)
{
    PopulateButtons(name + QString(".cal"));
}

void MainWindow::on_anyButtonClicked(QObject * obj)
{
    ButtonAction *action = (ButtonAction*)obj;
    if (!action->insert.isNull()) {
        ui->lineEdit->lineEdit()->insert(action->insert);
    }
    if (!action->set.isNull()) {
        ui->lineEdit->lineEdit()->setText(action->set);
    }
    if (action->send) {
        ui->plainTextEdit->appendPlainText("> "+ui->lineEdit->currentText().trimmed());
        QScriptValue result = scriptEngine->evaluate(QString(ui->lineEdit->currentText()));
        if (result.isError()) {
            ui->statusBar->showMessage(result.toString());
        } else {
            ui->plainTextEdit->appendPlainText(result.toString());
            ui->lineEdit->insertItem(0,result.toString());
            ui->lineEdit->setCurrentIndex(0);
            ui->statusBar->showMessage("");
        }
    }
}

void MainWindow::on_lineEdit_returnPressed()
{
    static ButtonAction send(NULL, NULL, true);
    on_anyButtonClicked(&send);
}

void MainWindow::on_actionQuit_triggered()
{
    this->close();
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox msgBox;
    msgBox.setTextFormat(Qt::RichText);
    msgBox.about(this,
                 "About",
                 "<b>PolyCalc v" + version + "</b><br />" +
                 "Configurable UI Calculator<br />" +
                 QString::fromUtf8("© 2012 PolyCalc Development Team<br />") +
                 "Distributed under GNU GPL v3<br />" +
                 "<a href='http://"+prjUrl+"'>"+prjUrl+"</a>");
}

void MainWindow::on_actionHelp_triggered()
{
    QMessageBox msgBox;
    msgBox.setTextFormat(Qt::RichText);
    msgBox.setText("For help please visit PolyCalc Help page:<br />"
                   "<a href='http://"+helpUrl+"'>"+helpUrl+"</a>");
    msgBox.exec();
}

void MainWindow::on_actionCommand_Log_triggered()
{
    ui->dockWidget->setVisible(ui->actionCommand_Log->isChecked());
}

